@@ -1,23 +1,39 @@
 import os
-from time import time
+import logging
 
+import dbt.main
+import dbt.logger
+
+from importlib import reload
 from IPython.core.magic import line_cell_magic, line_magic, magics_class, needs_local_scope
 from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring
-from pyspark.sql import SparkSession
-import pyspark.sql.functions as F
-
+from jupyterlab_sql_editor.ipython.sparkdf import display_df
 from jupyterlab_sql_editor.ipython_magic.common.base import Base
 from jupyterlab_sql_editor.ipython_magic.sparksql.spark_export import update_database_schema, update_local_database
-from traitlets import Unicode
+from pyspark.sql import SparkSession
+from time import time
+from traitlets import List, Unicode
 
+VALID_OUTPUTS = ['sql', 'text', 'json', 'html', 'grid', 'schema', 'skip', 'none']
 
-from jupyterlab_sql_editor.ipython.sparkdf import display_df
 
 @magics_class
 class SparkSql(Base):
-    valid_outputs = ['sql','text','json','html','grid','schema','skip','none']
-    dbt_args = []
-    dbt_project_dir = ''
+    dbt_args = List([], config=True, help='dbt arguments')
+    dbt_project_dir = Unicode('', config=True, help='Path to dbt project directory')
+
+    def __init__(self, shell=None, line=None, local_ns=None, **kwargs):
+        super().__init__(shell, **kwargs)
+        self.args = parse_argstring(self.sparksql, line)
+        self.spark = None
+        self.output = self.args.output.lower()
+        if self.outputFile is None:
+            self.outputFile = f"{os.path.expanduser('~')}/.local/sparkdb.schema.json"
+        if self.args.truncate and self.args.truncate > 0:
+            self.truncate = self.args.truncate
+        else:
+            self.truncate = 256
+        self.set_user_ns(local_ns)
 
     @needs_local_scope
     @line_cell_magic
@@ -25,31 +41,21 @@ class SparkSql(Base):
     @argument('sql', nargs='*', type=str, help='SQL statement to execute')
     @argument('-l', '--limit', metavar='max_rows', type=int, help='The maximum number of rows to display. A value of zero is equivalent to `--output skip`')
     @argument('-r', '--refresh', metavar='all|local|none', type=str, default='none',
-                help='Force the regeneration of the schema cache file. The `local` option will only update tables/views created in the local Spark context.')
+              help='Force the regeneration of the schema cache file. The `local` option will only update tables/views created in the local Spark context.')
     @argument('-d', '--dataframe', metavar='name', type=str, help='Capture dataframe in a local variable named `name`')
     @argument('-c', '--cache', action='store_true', help='Cache dataframe')
     @argument('-e', '--eager', action='store_true', help='Cache dataframe with eager load')
     @argument('-v', '--view', metavar='name', type=str, help='Create or replace a temporary view named `name`')
     @argument('-o', '--output', metavar='sql|json|html|grid|text|schema|skip|none', type=str, default='html',
-                help='Output format. Defaults to html. The `sql` option prints the SQL statement that will be executed (useful to test jinja templated statements)')
+              help='Output format. Defaults to html. The `sql` option prints the SQL statement that will be executed (useful to test jinja templated statements)')
     @argument('-s', '--show-nonprinting', action='store_true', help='Replace none printable characters with their ascii codes (LF -> \x0a)')
     @argument('-j', '--jinja', action='store_true', help='Enable Jinja templating support')
     @argument('-b', '--dbt', action='store_true', help='Enable DBT templating support')
     @argument('-t', '--truncate', metavar='max_cell_length', type=int, help='Truncate output')
-    def sparksql(self, line=None, cell=None, local_ns=None):
+    def sparksql(self, cell=None, shell=None, line=None, local_ns=None):
         "Magic that works both as %sparksql and as %%sparksql"
-
-        self.set_user_ns(local_ns)
-        args = parse_argstring(self.sparksql, line)
-        output_file = self.outputFile or f"{os.path.expanduser('~')}/.local/sparkdb.schema.json"
-        output = args.output.lower()
-
-        truncate = 256
-        if args.truncate and args.truncate > 0:
-            truncate = args.truncate
-
-        if not output in self.valid_outputs:
-            print(f'Invalid output option {args.output}. The valid options are [sql|json|text|html|grid|skip|none].')
+        if not self.output in VALID_OUTPUTS:
+            print(f'Invalid output option {self.args.output}. The valid options are [sql|json|text|html|grid|skip|none].')
             return
 
         self.spark = self.get_instantiated_spark_session()
@@ -58,23 +64,18 @@ class SparkSql(Base):
             return
 
         catalog_array = self.get_catalog_array()
-        if args.refresh.lower() == 'all':
-            update_database_schema(self.spark, output_file, catalog_array)
+        if self.check_refresh(self.args.refresh, self.outputFile, catalog_array):
             return
-        elif args.refresh.lower() == 'local':
-            update_local_database(self.spark, output_file)
-            return
-        elif args.refresh.lower() != 'none':
-            print(f'Invalid refresh option given {args.refresh}. Valid refresh options are [all|local|none]')
 
-        if args.dbt:
-            sql = self.get_dbt_sql_statement(cell, args.sql)
+        if self.args.dbt:
+            sql = self.get_dbt_sql_statement(cell, self.args.sql)
         else:
-            sql = self.get_sql_statement(cell, args.sql, args.jinja)
-            
+            sql = self.get_sql_statement(cell, self.args.sql, self.args.jinja)
+
         if not sql:
             return
-        elif output == 'sql':
+
+        if self.output == 'sql':
             return self.display_sql(sql)
 
         # statements like INSERT INTO, USE SCHEMA, CREATE TABLE, DROP TABLE
@@ -89,33 +90,43 @@ class SparkSql(Base):
             print(f"Execution time: {elapsed:.2f} seconds")
             return
 
-        if args.cache or args.eager:
-            load_type = 'eager' if args.eager else 'lazy'
+        if self.args.cache or self.args.eager:
+            load_type = 'eager' if self.args.eager else 'lazy'
             print(f'Cached dataframe with {load_type} load')
             result = result.cache()
-            if args.eager:
+            if self.args.eager:
                 result.count()
-        if args.view:
-            print(f'Created temporary view `{args.view}`')
-            result.createOrReplaceTempView(args.view)
-        if args.dataframe:
-            print(f'Captured dataframe to local variable `{args.dataframe}`')
-            self.shell.user_ns.update({args.dataframe: result})
+        if self.args.view:
+            print(f'Created temporary view `{self.args.view}`')
+            result.createOrReplaceTempView(self.args.view)
+        if self.args.dataframe:
+            print(f'Captured dataframe to local variable `{self.args.dataframe}`')
+            self.shell.user_ns.update({self.args.dataframe: result})
 
-
-        limit = args.limit
+        limit = self.args.limit
         if limit is None:
             limit = self.limit
 
-        if limit <= 0 or output == 'skip' or output == 'none':
+        if limit <= 0 or self.output == 'skip' or self.output == 'none':
             print('Query execution skipped')
             return
 
-        if output == 'schema':
+        if self.output == 'schema':
             result.printSchema()
             return
 
-        display_df(result, output=output, limit=limit, truncate=truncate, show_nonprinting=args.show_nonprinting)
+        display_df(result, output=self.output, limit=limit, truncate=self.truncate, show_nonprinting=self.args.show_nonprinting)
+
+    def check_refresh(self, refresh_arg, output_file, catalog_array):
+        if refresh_arg.lower() == 'all':
+            update_database_schema(self.spark, output_file, catalog_array)
+            return True
+        if self.args.refresh.lower() == 'local':
+            update_local_database(self.spark, output_file)
+            return True
+        if refresh_arg.lower() != 'none':
+            print(f'Invalid refresh option given {refresh_arg}. Valid refresh options are [all|local|none]')
+        return False
 
     @staticmethod
     def get_instantiated_spark_session():
@@ -127,38 +138,30 @@ class SparkSql(Base):
             sql = ' '.join(sql_argument)
         if not sql:
             print('No sql statement to execute')
-            return
-        
-        stage_file = self.dbt_project_dir + "/analyses/__sparksql__stage_file__.sql"
-        with open(stage_file, "w") as f:
-            f.write(sql)
+            return None
+
+        stage_file_path = self.dbt_project_dir + "/analyses/__sparksql__stage_file__.sql"
+        with open(stage_file_path, "w", encoding="utf8") as stage_file:
+            stage_file.write(sql)
 
         dbt_compile_args = [
-                '--no-write-json',
-                'compile', 
-                '--model', 
-                '__sparksql__stage_file__',
-                ] + self.dbt_args
+            '--no-write-json',
+            'compile',
+            '--model',
+            '__sparksql__stage_file__',
+            ] + self.dbt_args
         results, succeeded = self.invoke_dbt(dbt_compile_args)
-        os.remove(stage_file)
+        os.remove(stage_file_path)
         if succeeded:
-            compiled_file = self.dbt_project_dir + "/" + results.results[0].node.compiled_path
-            with open(compiled_file, "r") as f:
-                compiled_sql = f.read()
+            compiled_file_path = self.dbt_project_dir + "/" + results.results[0].node.compiled_path
+            with open(compiled_file_path, "r", encoding="utf8") as compiled_file:
+                compiled_sql = compiled_file.read()
             return compiled_sql
         return ""
 
-    def import_dbt(self):
-        import importlib
-        if not importlib.util.find_spec("dbt.main"):
-            print('dbt is not installed\npip install dbt-core')
-            return False
-    
+    @staticmethod
+    def import_dbt():
         # reset dbt logging to prevent duplicate log entries.
-        from importlib import reload
-        import logging
-        import dbt.main
-        import dbt.logger
         reload(dbt.main)
         reload(dbt.logger)
         logger = logging.getLogger("configured_std_out")
@@ -168,17 +171,17 @@ class SparkSql(Base):
 
     def invoke_dbt(self, args):
         if self.import_dbt():
-            import dbt.main
             return dbt.main.handle_and_check(args)
+        return None
 
     def get_dbt_project_dir(self, args):
         if self.import_dbt():
-            import dbt.main
-            parsed = dbt.main.parse_args(['debug'] + self.dbt_args)
+            parsed = dbt.main.parse_args(args)
             return parsed.project_dir
+        return None
 
     @line_magic
-    def dbt(self, line=None, local_ns=None):
+    def dbt(self, line=None):
         self.dbt_args = line.split()
         self.dbt_project_dir = self.get_dbt_project_dir(['debug'] + self.dbt_args)
         if not self.dbt_project_dir:
@@ -186,7 +189,4 @@ class SparkSql(Base):
             return
         os.chdir(self.dbt_project_dir)
         self.invoke_dbt(['debug'] + self.dbt_args)
-
-
-
-
+        return
